/**
 * @fileOverview
 * @name firewall.js
 * @author Thomas Del Grande, Roberto Cordoni
 */

/**
 * @class FirewallController
 * @param {Object}
 *          $http - AngularJS service for HTTP communication.
 * @param {Object}
 *          $log - AngularJS service for logging.
 * @param {Object}
 *          $anchorScroll - AngularJS service for scroll page.
 *
 * @property {Array} rules - Array of existing rules
 * @property {String} error - Text containing last error encountered
 * @property {Boolean} valid_src - Indicates whether the src parameter is valid
 * @property {Boolean} valid_dst - Indicates whether the dst parameter is valid
 * @property {Boolean} valid_dport_start - Indicates whether the dport start parameter is valid
 * @property {Boolean} valid_dport_end - Indicates whether the dport end parameter is valid
 * @property {String} firewall_state - Can be either 'enabled' or 'disabled'.
 * @property {Boolean} port_selection_enabled - Indicates whether the port fields are to be enabled or disabled.
 */

angular.module('ag561app').controller('FirewallController',
        ['$http', '$log', function($http, $log) {

  var ctrl = this;
  ctrl.rules = [];
  ctrl.error = "";

  ctrl.firewall_state = false;
  ctrl.port_selection_enabled = true; /* enable / disable port configuration depending on protocol for rule */
  ctrl.loading = false;
  ctrl.invalid_config = false;

  /**
  * Clear alert message from web page, but do this only
  * if we aren't with some error. We just want to erase
  * the message of danger if all the variables are correct.
  *
  * @function clear_alert_messages
  * @memberof FirewallController
  */
  ctrl.clear_alert_messages = function() {
      if (!ctrl.invalid_config) {
        ctrl.alert_error = false;
        ctrl.alert_success = false;
      }
  };

  /**
   * Get firewall state, rules, etc.
   *
   * @function get
   * @memberof FirewallController
   */
  ctrl.get = function() {
    ctrl.loading = true;
    $http.get('/api/config/firewall')
      .then(function(response) {
          // First function handles success
          ctrl.rules = response.data.rules;
          ctrl.interfaces = response.data.interfaces;
          /* firewall_state is translated to boolean for toogle to work */
          ctrl.firewall_state = response.data.firewall_state == "enabled";
          ctrl.loading = false;
      }, function(response) {
          // Second function handles error
          ctrl.error = response.data;
          ctrl.loading = false;
      });
  }

  ctrl.get();

  /**
   * Save and apply the configuration by sending a PUT message
   * to the device.
   *
   * @function apply_and_save
   * @memberof FirewallController
   */
  ctrl.save = function() {
    ctrl.loading = true;

    /* Delete JS only variables before API PUT */
    for (var i = 0; i < ctrl.rules.length; i++) {
        delete ctrl.rules[i].invalid_src;
        delete ctrl.rules[i].invalid_name;
        delete ctrl.rules[i].invalid_dport_start;
        delete ctrl.rules[i].invalid_dport_end;
        delete ctrl.rules[i].invalid_config;
    }

    data = { 'rules' : ctrl.rules };

    /* firewall_state is translated from boolean to string */
    data.firewall_state = ctrl.firewall_state ? "enabled" : "disabled";

    $http.put('/api/config/firewall', data)
      .then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
        ctrl.get();
      }, function(response) {
        // Some error happened then print this in web page
        ctrl.loading = false;
      });
  };

  /**
   * Add a new firewall rule on the INPUT chain.
   *
   * @function add_rule
   * @memberof FirewallController
   */
  ctrl.rule_add = function() {
      var new_rule = {
          'name': '',
          'in_interface': 'any',
          'src': 'any',
          'protocol': 'any',
          'dport_start': '',
          'dport_end': '',
          'invalid_name': true,
          'invalid_src': false,
          'invalid_dport_start': false,
          'invalid_dport_end': false,
          'invalid_config': true
      };
      ctrl.port_selection_enabled = false;
      ctrl.rules.push(new_rule);
      ctrl.check_invalid_config();
      ctrl.edit_index = ctrl.rules.indexOf(new_rule);
  };


    /**
     * Delete a rule from the rules list
     *
     * @function rule_delete
     * @memberof FirewallController
     */
    ctrl.rule_delete = function(index) {
        ctrl.rules.splice(index, 1);
        ctrl.check_invalid_config();
    };


  /**
   * Check if input fields are valid.
   *
   * @function is_port_valid
   * @memberof FirewallController
   * @returns True if ALL the fields are valid. False otherwise.
   */
  function is_port_valid(port) {
      if (port > 0 && port < 65536)
          return true;
      return false;
  }

  /**
   * Check if IP address/network/mask string is valid
   *
   * @returns true if yes, false if not.
   */
  function is_ip_valid(ip) {
      var network = ip.split("/")[0];
      var mask = ip.split("/")[1];
      var networkformat = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$");
      var maskformat = new RegExp("^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)|(3[0-2]|[1-2]?[0-9]))$");

      if (typeof(network) != 'string' || !network.match(networkformat))
          return false;

      if (typeof(mask) == 'string' && !mask.match(maskformat))
          return false;

      return true;
  }

  /**
   * Check if name is valid and not duplicated.
   */
  ctrl.check_rule_name = function(rule) {
      var count = 0;
      /* Check if name is valid */
      var pattern = /^\w+$/;
      if (pattern.test(rule.name)) {
          /* Check for duplicates */
          for (var i = 0; i < ctrl.rules.length; i++) {
              if (ctrl.rules[i].name == rule.name)
                  count++;
          }
      }
      rule.invalid_name = count != 1;

      ctrl.check_invalid_config();
  };

  ctrl.check_source = function(rule) {
    /* Network check */
    if (rule.src == "any") {
        rule.invalid_src = false;
    } else if (rule.src.length > 0) {
        rule.invalid_src = !is_ip_valid(rule.src);
    } else {
        rule.invalid_src = true;
    }

    ctrl.check_invalid_config();
  };

  ctrl.check_invalid_config = function() {
      for (var i = 0; i < ctrl.rules.length; i++) {
          var rule = ctrl.rules[i];
          if (rule.invalid_name || rule.invalid_src || rule.invalid_dport_start ||
              rule.invalid_dport_end) {
                rule.invalid_config = true;
                ctrl.invalid_config = true;
                ctrl.alert_error = true;
                return;
          }
          rule.invalid_config = false;
      }

      ctrl.invalid_config = false;
      ctrl.alert_error = false;
  };

  /**
   * Update port configuration enabled / disabled class.
   * If TCP and/or UDP is selected, then ports can be configured.
   *
   * @function update_port_selection
  */
  ctrl.update_port_selection = function(rule) {
      if (rule.protocol == "tcp" || rule.protocol == "udp") {
          rule.invalid_dport_start = rule.dport_start.length == 0;
          ctrl.port_selection_enabled = true;
      } else {
          ctrl.port_selection_enabled = false;
          rule.invalid_dport_start = false;
          rule.dport_start = "";
          rule.dport_end = "";
      }
      ctrl.check_invalid_config();
  };

  ctrl.check_dports = function(rule) {
      /* Ports check */
      if (rule.dport_start.length == 0) {
          /* Blank port is only valid for ANY or ICMP protocols */
          rule.invalid_dport_start = (rule.protocol == "any" || rule.protocol == "icmp");
          rule.invalid_dport_end = rule.dport_end.length > 0;
      } else {
          /* Check if port start is within accepted range */
          var dport_start = parseInt(rule.dport_start);
          rule.invalid_dport_start = !is_port_valid(dport_start);
          /* Check if port end */
          if (!rule.dport_end || rule.dport_end.length == 0) {
              /* Port end may be blank */
              rule.invalid_dport_end = false;
          } else {
              /* Check if port end is within accepted range */
              var dport_end = parseInt(rule.dport_end);
              rule.invalid_dport_end = !is_port_valid(dport_end) || dport_end < dport_start;
          }
      }

      ctrl.check_invalid_config();
  };

}]);
